classdef A255 < Singleton
    %A255 Controller for the CRS A255 robotic arm
    %
    %
    %   Burton Ma
    
    properties (Access = private)
        serialPort = [];
        namedVars = containers.Map;
    end
    
    methods(Access=private)
        % Guard the constructor against external invocation.  We only want
        % to allow a single instance of this class.  See description in
        % Singleton superclass.
        function newObj = A255()
            s = serial('/dev/ttyS0', 'BaudRate', 57600, 'InputBufferSize', 1024);
            fopen(s);
            % fopen will throw an exception if there is a problem
            newObj.serialPort = s;
            newObj.sendCommand('ash test');
        end
        
        function delete(obj)
            fclose(obj.serialPort);
        end
        
        function result = sendCommand(obj, cmd, ignore)
            %SENDCOMMAND Send a command string
            
            if nargin == 2
                ignore = false;
            end 
            
            %flush any data on the serial port
            b = obj.serialPort.BytesAvailable;
            while b > 0
                fscanf(obj.serialPort, '%c', b);
                b = obj.serialPort.BytesAvailable;
            end
            
            fprintf(obj.serialPort, cmd);
            pause(0.5);
            result = [];
            
            if ignore
                return
            end
            
            %read from serial port until the ash prompt is seen
            raw = [];
            lenraw = length(raw);
            while lenraw < 6 || strcmp(raw(end-5:end-1), 'test>') ~= 1
                b = obj.serialPort.BytesAvailable;
                if b > 0
                    txt = fscanf(obj.serialPort, '%c', b);
                    raw = [raw txt];
                    lenraw = length(raw);
                else
                    pause(0.1);   %wait for the serial port
                end
            end
                        
            %first part of result should be the command sent
            cmdlen = length(cmd);
            if cmdlen > length(raw)
                error('sendCommand:failed', ...
                      'serial port error, command failed to send');
            end
            if strcmp(raw(1:cmdlen), cmd) ~= 1
                 error('sendCommand:failed', ...
                      'serial port error, command failed to send');
            end
            raw(1:cmdlen) = [];
            
            %rest of raw is the result
            delim = char([13 10]);
            while length(raw) > 0
                [txt, rest] = strtok(raw, delim);
                result = strvcat(result, txt);
                raw = rest;
            end
            
            %most commands will have last row of result
            %starting with 'test>'; strip this off of the result
            last = strtrim(result(end, :));
            if strcmp(last, 'test>') == 1
                result(end, :) = [];
            end
        end
        
    end
    
    methods(Static)
        % Concrete implementation.  See Singleton superclass.
        function obj = instance()
            persistent uniqueInstance
            if isempty(uniqueInstance) | ~isvalid(uniqueInstance)
                obj = A255();
                uniqueInstance = obj;
            else
                obj = uniqueInstance;
            end
        end
    end
    
    methods
        function quit(obj)
            %QUIT Quit using the arm.
            %Use R.QUIT() to quit using the arm R.
            obj.sendCommand('exit', true);
            obj.sendCommand('yes', true);
            obj.delete();
        end
        
        function home(obj)
            %HOME Home the arm.
            %R.HOME() will home R.
            x = obj.sendCommand('home');
        end
           
        function ready(obj)
            %READY Move the arm the ready position.
            %R.READY() will move R to the ready position.
            obj.sendCommand('ready');
        end
        
        function result = joint(obj, J, deg)
            %JOINT Rotate a specified joint.
            %R.JOINT(J, DEG) will rotate joint J by DEG degrees.
            s = sprintf('%s %d, %f', 'joint ', J, deg);
            result = obj.sendCommand(s);
        end
        
        
        function [J, P, T, result] = w0(obj)
            %W0 Get the commanded joint angles and robot position.
            %J = R.W0() returns the commanded joint angles J in degrees.
            %
            %[J, P] = R.W0() also returns the world position P of the end
            %plate.
            %
            %[J, P, T] = R.W0() also returns the yaw, pitch, and roll
            %angles T of the end effector in degrees.
            %
            %[J, P, T, RESULT] = R.W0() also returns the raw text RESULT.
            
            result = obj.sendCommand('w0');
            J = [];
            P = [];
            T = [];
            %joint angles are on row 6
            jt = result(6, :);
            J = sscanf(jt, 'JOINTS %f %f %f %f %f')';
            %position and ZYX rotation of end effector are on row 10
            pos = result(10, :);
            PT = sscanf(pos, '%f %f %f %f %f %f');
            P = PT(1:3)';
            T = PT(4:6)';
        end
        
        function [J, P, T, result] = w4(obj)
            
            result = obj.sendCommand('w4');
            J = [];
            P = [];
            T = [];
        end
        
        
        function finish(obj)
            %FINISH Finish a movement command.
            %R.FINISH will wait for the arm to complete its current
            %motion before continuing.
            obj.sendCommand('finish');
        end
        
        function lock(obj, J)
            
            s = sprintf('%s %d', 'lock', J(1));
            if length(J) > 1
                s = [s sprintf(', %d', J(2:end))];
            end
            result = obj.sendCommand(s);
        end
        
        % ADD YOUR OWN FUNCTIONS BELOW
        function eraseall(obj, name)
            %ERASEALL Clears all named variables
            %R.ERASEALL will clear all named variables. You probably
            %want to call this function once before you issue any
            %HERE and MOVE commands.
            obj.sendCommand('eraseall', true);
            obj.sendCommand('yes', true);
            obj.namedVars = containers.Map;
        end
        
        function here(obj, name)
            %HERE Stores the current arm Cartesian coordinates
            %R.HERE(NAME) will create store the current arm Cartesian
            %coordinates in the variable named NAME. If the variable
            %does not exist, one will be created.
            %
            %Example:
            %
            %r.here('p1')
            %
            %Note that this function will fail if a variable named
            %NAME already exists in the ash shell and it does not have
            %Cartesian coordinate type. You should call ERASEALL before
            %using HERE for the first time.
            if isKey(obj.namedVars, name) == false
                s = sprintf('%s _%s', 'new', name);
                obj.sendCommand(s);
                obj.namedVars(name) = [];
            end
            s = sprintf('here %s', name);
            obj.sendCommand(s);
        end
        
        
        function move(obj, name)
            %MOVE Move the arm to a named position
            %R.MOVE(NAME) will move the arm to a named Cartesian position.
            %An error occurs if NAME does not exist (or if it exists but
            %does not have Cartesian coordinate type). See HERE for more
            %details.
            %
            %Example:
            %
            %r.move('p1')
            %
            if isKey(obj.namedVars, name) == false
                s = sprintf('Variable %s does not exist', name);
                error(s);
            end
            s = sprintf('move %s', name);
            obj.sendCommand(s);
        end
        
       
    end
end

